package com.dayou.brokerage;

import com.dayou.bo.BrokerageDetailBO;
import com.dayou.brokerage.config.LandBrokerageDataConfiguration;
import com.dayou.brokerage.factory.BrokerageCalculateFactory;
import com.dayou.brokerage.constants.BrokerageRule;
import com.dayou.brokerage.handler.LandMarketerBrokerageHandler;
import com.dayou.brokerage.handler.LandNonMarketerBrokerageHandler;
import com.dayou.brokerage.validator.BrokerageValidator;
import com.dayou.dto.ItemDTO;
import com.dayou.utils.DateUtils;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;

/**
 * 类说明：
 *
 * @author: wucl
 * @since: 2023/3/3
 * created with IntelliJ IDEA.
 */
@Component
public class BrokerageCalculateSupport implements BrokerageCalculateFactory,BrokerageCalculator{

    @Autowired
    private BrokerageValidator brokerageValidator;

    @Autowired
    @Qualifier("landMarketerBrokerageHandler")
    private LandMarketerBrokerageHandler landMarketerBrokerageHandler;

    @Autowired
    @Qualifier("landNonMarketerBrokerageHandler")
    private LandNonMarketerBrokerageHandler landNonMarketerBrokerageHandler;

    @Autowired
    private LandBrokerageDataConfiguration landBrokerageDataConfiguration;

    @Override
    public BrokerageCalculator getCalculator(BrokerageRule rule){
        switch (rule){
            case LAND_MARKETER_RULE:
                return landMarketerBrokerageHandler;
            case LAND_OTHER_RULE:
                return landNonMarketerBrokerageHandler;
        }
        return null;
    }
    /**
     * 校验项目提成状态
     * @param brokerageDetailBo
     * @return
     */
    protected void checkItemBrokerageStatus(BrokerageDetailBO brokerageDetailBo){
        brokerageValidator.checkItemBrokerageStatus(brokerageDetailBo);
    }

    /**
     * 市场人员预提校验
     * @param brokerageDetailBo
     * @return
     */
    protected void checkMarketerPredictBrokerage(BrokerageDetailBO brokerageDetailBo){
        @NotNull
        BigDecimal paymentTarget = brokerageDetailBo.getPaymentTarget();
        BigDecimal avgMonthTarget = paymentTarget.divide(landBrokerageDataConfiguration.monthOfYear, 2, RoundingMode.HALF_UP);
        int month = DateUtils.getMonth(new Date());
        //当月应完成回款金额
        BigDecimal brokerageTarget = avgMonthTarget.multiply(new BigDecimal(month));
        brokerageValidator.checkMarketerPredictBrokerage(brokerageDetailBo,brokerageTarget);
    }

    @Override
    public BigDecimal actualAmount(ItemDTO itemDTO) {
        return null;
    }

    @Override
    public BigDecimal aheadAmount(BigDecimal predictAmount) {
        return null;
    }

    @Override
    public BigDecimal advanceAmount() {
        return null;
    }

    @Override
    public BigDecimal residueAmount() {
        return null;
    }
}
