package com.dayou.brokerage.config;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.dayou.common.BaseEntity;
import com.dayou.entity.BrokerageMarketerRatio;
import com.dayou.enums.GlobalConfigEnum;
import com.dayou.enums.MergeKeyInterface;
import com.dayou.service.IBrokerageMarketerRatioService;
import com.dayou.service.IGlobalConfigService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;

import javax.annotation.Nullable;
import java.math.BigDecimal;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

import static com.dayou.common.Constants.MERGE_SYMBOL;

/**
 * 类说明：市场人员提成系数表
 *
 * @author: wucl
 * @since: 2023/3/6
 * created with IntelliJ IDEA.
 */
public class LandBrokerageDataConfiguration extends BrokerageMarketerRatio implements MergeKeyInterface {


    @Autowired
    private IBrokerageMarketerRatioService brokerageMarketerRatioService;

    @Autowired
    private IGlobalConfigService globalConfigService;

    /**
     * 由项目类型和项目来源决定的提成系数集合
     */
    private ConcurrentHashMap<String, BigDecimal> ratioList = new ConcurrentHashMap<String,BigDecimal>(256);

    /**
     * 每个人基础提成金额
     */
    private ConcurrentHashMap<Long,BigDecimal> userBaseBrokerageAmounts = new ConcurrentHashMap<>(256);

    @Nullable
    public BigDecimal marketerBrokeragePercentage ;

    @Nullable
    public BigDecimal monthOfYear;

    @Nullable
    public BigDecimal nonMarketerMonthBrokerage;

    @Nullable
    public BigDecimal participantBrokeragePercentage;


    public void loadRatioList(){
        List<BrokerageMarketerRatio> list = brokerageMarketerRatioService.list(new LambdaQueryWrapper<BrokerageMarketerRatio>()
                .eq(BaseEntity::getDeleted, Boolean.FALSE));
        list.stream().forEach(x->{
            ratioList.put(x.mergeKey(),new BigDecimal(x.getRatio().toString()));
        });

        nonMarketerMonthBrokerage = new BigDecimal(globalConfigService.getValue(GlobalConfigEnum.FINANCE.getCode(),
                GlobalConfigEnum.Finance.NON_MARKETER_MONTH_BROKERAGE.getCode()));

        monthOfYear = new BigDecimal(globalConfigService.getValue(GlobalConfigEnum.SYS.getCode(),
                GlobalConfigEnum.Sys.MONTH_OF_YEAR.getCode()));

        marketerBrokeragePercentage = new BigDecimal(globalConfigService.getValue(GlobalConfigEnum.FINANCE.getCode(),
                GlobalConfigEnum.Finance.MARKETER_BROKERAGE_PERCENTAGE.getCode()));

        participantBrokeragePercentage = new BigDecimal(globalConfigService.getValue(GlobalConfigEnum.FINANCE.getCode(),
                GlobalConfigEnum.Finance.PARTICIPANT_BROKERAGE_PERCENTAGE.getCode()));
    }

    public ConcurrentHashMap<String, BigDecimal> getRatioList() {
        return ratioList;
    }
}
