package com.dayou.brokerage.handler;

import com.dayou.bo.BrokerageDetailBO;
import com.dayou.brokerage.BrokerageCalculateSupport;
import com.dayou.brokerage.MarketerBrokerageCalculator;
import com.dayou.brokerage.config.LandBrokerageDataConfiguration;
import com.dayou.dto.ItemDTO;
import com.dayou.exception.BusinessException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;

/**
 * 类说明：土地规划部市场人员提成计算处理器
 *
 * @author: wucl
 * @since: 2023/3/6
 * created with IntelliJ IDEA.
 */
@Component
@Slf4j
public class LandMarketerBrokerageHandler extends BrokerageCalculateSupport implements MarketerBrokerageCalculator {

    @Autowired
    private LandBrokerageDataConfiguration landBrokerageDataConfiguration;

    @Override
    public BrokerageDetailBO predictPersonalAmount(BrokerageDetailBO bo) {
        String mergeKey = bo.mergeKey();
        BigDecimal ratio = landBrokerageDataConfiguration.getRatioList().get(mergeKey);
        bo.setRate(ratio);
        if (ratio !=null){
            checkItemBrokerageStatus(bo);
            try {
                checkMarketerPredictBrokerage(bo);
                bo.setPredictAmount(bo.getAmount().multiply(ratio));
                bo.setAheadAmount(aheadAmount(bo.getPredictAmount()));
                bo.setMarketerAdvanceBrokerageAble(Boolean.TRUE);
            } catch (BusinessException e) {
                log.info("项目id:[{}],客户经理id：[{}],[{}]",bo.getId(),bo.getUserId(),e.getMessage());
            }
        }else {
            log.info("项目id:[{}],合同金额：[{}],提成系数：[null],[未设置提成系数，无法计算客户经理提成]",bo.getId(),bo.getAmount());
        }
        return bo;

    }

    @Override
    public BigDecimal aheadAmount(BigDecimal predictAmount){
        BigDecimal aheadAmount = predictAmount.multiply(landBrokerageDataConfiguration.marketerBrokeragePercentage);
        return aheadAmount;
    }
    @Override
    public BigDecimal advanceAmount() {
        return null;
    }
    @Override
    public BrokerageDetailBO predictTeamShareAmount(BrokerageDetailBO bo) {
        return null;
    }
    @Override
    public BigDecimal actualAmount(ItemDTO itemDTO) {
        return null;
    }

    @Override
    public BigDecimal residueAmount() {
        return null;
    }


}
