package com.dayou.brokerage.handler;

import cn.hutool.core.bean.BeanUtil;
import com.dayou.bo.BrokerageDetailBO;
import com.dayou.brokerage.BrokerageCalculateSupport;
import com.dayou.brokerage.NonMarketerMarketerBrokerageCalculator;
import com.dayou.brokerage.config.LandBrokerageDataConfiguration;
import com.dayou.brokerage.validator.BrokerageValidator;
import com.dayou.dto.ItemDTO;
import com.dayou.entity.ItemBrokerageDetail;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;

/**
 * 类说明：土地规划部 非 市场人员提成计算处理器
 *
 * @author: wucl
 * @since: 2023/3/8
 * created with IntelliJ IDEA.
 */
@Component
@Slf4j
public class LandNonMarketerBrokerageHandler extends BrokerageCalculateSupport implements NonMarketerMarketerBrokerageCalculator {

    @Autowired
    private BrokerageValidator brokerageValidator;

    @Autowired
    private LandBrokerageDataConfiguration landBrokerageDataConfiguration;

    @Override
    public ItemBrokerageDetail predictAmount(BrokerageDetailBO brokerageDetailBO) {
        brokerageValidator.checkItemBrokerageStatus(brokerageDetailBO);
        ItemBrokerageDetail itemBrokerageDetail = new ItemBrokerageDetail();
        if (landBrokerageDataConfiguration.participantBrokeragePercentage!=null){
            //合同金额
            BigDecimal predictAmount = brokerageDetailBO.getAmount()
                    //参与项目提成总比例
                    .multiply(landBrokerageDataConfiguration.participantBrokeragePercentage)
                    //阶段权重
                    .multiply(brokerageDetailBO.getStageWeight())
                    //参与人权重
                    .multiply(brokerageDetailBO.getParticipantWeight());
            brokerageDetailBO.setPredictAmount(predictAmount.setScale(2,BigDecimal.ROUND_HALF_UP));
            BeanUtil.copyProperties(brokerageDetailBO,itemBrokerageDetail);
        }
        log.info("项目id:[{}],合同金额：[{}],提成系数：[null],[未设置[土规部参与项目整体提成比例]，无法计算参与人员提成]",brokerageDetailBO.getItemId(),brokerageDetailBO.getAmount());
        return itemBrokerageDetail;
    }

    @Override
    public BigDecimal aheadAmount(BigDecimal predictAmount){
        //todo
        return null;
    }
    @Override
    public BigDecimal actualAmount(ItemDTO itemDTO) {
        return null;
    }

    @Override
    public BigDecimal advanceAmount() {
        return null;
    }

    @Override
    public BigDecimal residueAmount() {
        return null;
    }

    @Override
    public BigDecimal advanceTotalAmount() {
        return null;
    }

    @Override
    public int advanceTimes() {
        return 0;
    }
}
