package com.dayou.brokerage.interceptor;

import com.dayou.brokerage.annotation.DoBrokerage;
import com.dayou.dto.ItemDTO;
import com.dayou.enums.OperationTypeEnum;
import com.dayou.mapper.ItemMapper;
import com.dayou.service.IItemBrokerageGeneralService;
import com.dayou.service.IItemService;
import com.dayou.service.IItemStageService;
import com.dayou.vo.ItemStageVO;
import lombok.extern.slf4j.Slf4j;
import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.lang.reflect.Method;

import static com.dayou.brokerage.constants.BrokerageRule.LAND_MARKETER_RULE;
import static com.dayou.brokerage.constants.BrokerageRule.LAND_OTHER_RULE;
import static com.dayou.enums.OperationTypeEnum.*;

/**
 * 类说明：提成切面
 *
 * @author: wucl
 * @since: 2023/3/7
 * created with IntelliJ IDEA.
 */
@Slf4j
@Component
public class BrokerageInterceptor implements MethodInterceptor {

    @Autowired
    private IItemBrokerageGeneralService iItemBrokerageGeneralService;

    @Override
    public Object invoke(MethodInvocation methodInvocation) throws Throwable {
        Method method = methodInvocation.getMethod();
        DoBrokerage annotation = method.getAnnotation(DoBrokerage.class);
        Object result = methodInvocation.proceed();
        if (annotation!=null && annotation.rule() == LAND_MARKETER_RULE && annotation.operation() == ADD){
            if (result instanceof Long){
                iItemBrokerageGeneralService.addMarketerBrokerage((Long) result);
                log.info("created general and marketer detail,itemId[{}]", result);
            };
        }
        if (annotation!=null && annotation.rule() == LAND_MARKETER_RULE && annotation.operation() == EDIT){
            ItemDTO itemDTO = (ItemDTO) methodInvocation.getArguments()[0];
            iItemBrokerageGeneralService.editMarketerBrokerage(itemDTO);
            log.info("updated general and marketer detail,itemId[{}]", itemDTO.getId());
        }
        if (annotation!=null && annotation.rule() == LAND_MARKETER_RULE && annotation.operation() == DELETE){
            Long itemId = (Long) methodInvocation.getArguments()[0];
            iItemBrokerageGeneralService.removeMarketerBrokerage(itemId);
            log.info("deleted general and marketer and non-marketer detail,itemId[{}]", itemId);
        }
        if (annotation!=null && annotation.rule() == LAND_OTHER_RULE && annotation.operation() == ADD){
            if (result instanceof Long){
                ItemStageVO itemStageVO = (ItemStageVO) methodInvocation.getArguments()[0];
                iItemBrokerageGeneralService.addParticipatorBrokerage(itemStageVO,(Long) result);
                log.info("created participator detail,itemId[{}]", itemStageVO.getItemId());
            };
        }
        if (annotation!=null && annotation.rule() == LAND_OTHER_RULE && annotation.operation() == EDIT){
            if (result instanceof Boolean && (Boolean) result){
                ItemStageVO itemStageVO = (ItemStageVO) methodInvocation.getArguments()[0];
                iItemBrokerageGeneralService.editParticipatorBrokerage(itemStageVO);
                log.info("created participator detail,itemId[{}]", itemStageVO.getItemId());
            };
        }
        if (annotation!=null && annotation.rule() == LAND_OTHER_RULE && annotation.operation() == DELETE){
            if (result instanceof Boolean && (Boolean) result){
                Long itemStageId = (Long) methodInvocation.getArguments()[0];
                iItemBrokerageGeneralService.removeParticipatorBrokerage(itemStageId);
                log.info("deleted non-marketer detail,itemId[{}]", itemStageId);
            };
        }
        return result;
    }
}
