package com.dayou.service.impl;

import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.dayou.bo.BrokerageDetailBO;
import com.dayou.brokerage.BrokerageCalculateSupport;
import com.dayou.brokerage.BrokerageCalculator;
import com.dayou.brokerage.MarketerBrokerageCalculator;
import com.dayou.brokerage.NonMarketerMarketerBrokerageCalculator;
import com.dayou.brokerage.constants.BrokerageMode;
import com.dayou.brokerage.constants.BrokerageRule;
import com.dayou.brokerage.constants.BrokerageState;
import com.dayou.brokerage.factory.BrokerageCalculateFactory;
import com.dayou.common.BaseEntity;
import com.dayou.dto.ItemDTO;
import com.dayou.entity.*;
import com.dayou.mapper.ItemBrokerageGeneralMapper;
import com.dayou.mapper.ItemMapper;
import com.dayou.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.vo.ItemStageVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;

import java.math.BigDecimal;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.transaction.annotation.Transactional;

/**
 * <p>
 * 项目提成总表 服务实现类
 * </p>
 *
 * @author wucl
 * @since 2023-03-03
 */
@Slf4j
@Service
public class ItemBrokerageGeneralServiceImpl extends ServiceImpl<ItemBrokerageGeneralMapper, ItemBrokerageGeneral> implements IItemBrokerageGeneralService {


    @Autowired
    private BrokerageCalculateSupport brokerageCalculateSupport;

    @Autowired
    private IUserTargetService userTargetService;

    @Autowired
    private IItemBrokerageDetailService iItemBrokerageDetailService;

    @Autowired
    private IMarketStatService marketStatService;

    @Autowired
    private ItemMapper itemMapper;

    @Override
    @SuppressWarnings("unchecked")
    public Page<ItemBrokerageGeneral> selectPage(Page page,ItemBrokerageGeneral itemBrokerageGeneral){
        return this.page(page, new QueryWrapper<ItemBrokerageGeneral>(itemBrokerageGeneral));
    }


    @Override
    public ItemBrokerageGeneral detail(Long id){
        return this.getById(id);
    }

    @Override
    public Boolean add(ItemBrokerageGeneral itemBrokerageGeneral){
        return  this.save(itemBrokerageGeneral);
    }

    @Override
    public Boolean update(ItemBrokerageGeneral itemBrokerageGeneral){
        return  this.updateById(itemBrokerageGeneral);
    }

    @Override
    public Boolean delete(Long id){
        //逻辑删除
        return this.removeById(id);
    }

    @Transactional
    @Override
    public void doCreateBrokerageGeneralAndDetail(BrokerageDetailBO bo) {
        MarketerBrokerageCalculator calculator = (MarketerBrokerageCalculator) brokerageCalculateSupport.getCalculator(BrokerageRule.LAND_MARKETER_RULE);
        Long userId = bo.getUserId();
        UserTarget userTarget = getUserTarget(userId);
        BigDecimal currentMonthPayment = marketStatService.currentYearPayment(userId);
        BigDecimal paymentTarget = new BigDecimal(userTarget.getPaymentTarget());
        bo.setCurrentPayment(currentMonthPayment);
        bo.setPaymentTarget(paymentTarget);
        bo.setItemBrokerageStatus(BrokerageState.NOT_START.getCode());
        bo = calculator.predictPersonalAmount(bo);
        ItemBrokerageGeneral general = ItemBrokerageGeneral.builder()
                .itemId(bo.getItemId())
                .marketerBrokerageAmount(bo.getPredictAmount())
                .marketerAdvanceBrokerageAble(bo.getMarketerAdvanceBrokerageAble())
                .brokeragePredictAmount(bo.getPredictAmount())
                .brokerageActualAmount(BigDecimal.ZERO)
                .brokerageRate(bo.getRate()).build();
        this.save(general);
        bo.setGeneralId(general.getId());
        iItemBrokerageDetailService.save(bo);
    }
    @Override
    public void addMarketerBrokerage(Long id) {
        Item item = itemMapper.selectById(id);
        if (item.getAmount()!=null){
            BrokerageDetailBO brokerageDetailBO = BrokerageDetailBO.builder().amount(item.getAmount()).cate(item.getCate()).businessSource(item.getBusinessSource()).itemId(id).build();
            brokerageDetailBO.setBrokerageRule(BrokerageRule.LAND_MARKETER_RULE.getCode());
            brokerageDetailBO.setBrokerageMode(BrokerageMode.PERSONAL.getCode());
            brokerageDetailBO.setAdvanceAmount(BigDecimal.ZERO);
            brokerageDetailBO.setPredictAmount(BigDecimal.ZERO);
            brokerageDetailBO.setActualAmount(BigDecimal.ZERO);
            brokerageDetailBO.setAheadAmount(BigDecimal.ZERO);
            brokerageDetailBO.setUserId(item.getUserId());
            this.doCreateBrokerageGeneralAndDetail(brokerageDetailBO);
        }else {
            log.info("未设置合同金额，创建提成总表和客户经理提成记录终止。itemId[{}]",id);
        }

    }

    @Transactional
    @Override
    public void editMarketerBrokerage(ItemDTO itemDTO) {
        ItemBrokerageGeneral general = this.getOne(new LambdaQueryWrapper<ItemBrokerageGeneral>().eq(ItemBrokerageGeneral::getItemId, itemDTO.getId()).eq(BaseEntity::getDeleted,Boolean.FALSE));
        if (general == null){
            addMarketerBrokerage(itemDTO.getId());
            log.info("非正常情况下，更新项目信息时创建了相关的提成记录");
            return;
        }
        ItemBrokerageDetail marketerDetail = iItemBrokerageDetailService.getOne(new LambdaQueryWrapper<ItemBrokerageDetail>().eq(ItemBrokerageDetail::getGeneralId, general.getId()).eq(BaseEntity::getDeleted, Boolean.FALSE));
        MarketerBrokerageCalculator calculator = (MarketerBrokerageCalculator) brokerageCalculateSupport.getCalculator(BrokerageRule.LAND_MARKETER_RULE);
        UserTarget userTarget = getUserTarget(marketerDetail.getUserId());
        BigDecimal currentMonthPayment = marketStatService.currentMonthPayment(marketerDetail.getUserId());
        BigDecimal paymentTarget = new BigDecimal(userTarget.getPaymentTarget());
        BrokerageDetailBO brokerageDetailBO = BrokerageDetailBO.builder().amount(itemDTO.getAmount()).cate(itemDTO.getCate()).businessSource(itemDTO.getBusinessSource()).itemId(itemDTO.getId()).paymentTarget(paymentTarget).currentPayment(currentMonthPayment).build();
        brokerageDetailBO.setUserId(marketerDetail.getUserId());
        brokerageDetailBO.setId(marketerDetail.getId());
        brokerageDetailBO = calculator.predictPersonalAmount(brokerageDetailBO);
        iItemBrokerageDetailService.update(brokerageDetailBO);
        general.setMarketerBrokerageAmount(brokerageDetailBO.getPredictAmount());
        general.setMarketerAdvanceBrokerageAble(brokerageDetailBO.getMarketerAdvanceBrokerageAble());
        general.setBrokerageRate(brokerageDetailBO.getRate());
        general.setBrokeragePredictAmount(brokerageDetailBO.getPredictAmount().add(general.getParticipantPredictBrokerageAmount()));
        this.update(general);
    }

    @Transactional
    @Override
    public void removeMarketerBrokerage(Long itemId) {
        ItemBrokerageGeneral general = this.getOne(new LambdaQueryWrapper<ItemBrokerageGeneral>().eq(ItemBrokerageGeneral::getItemId, itemId));
        this.removeById(general.getId());
        iItemBrokerageDetailService.remove(new LambdaQueryWrapper<ItemBrokerageDetail>().eq(ItemBrokerageDetail::getGeneralId,general.getId()));
    }

    @Transactional
    @Override
    public void addParticipatorBrokerage(ItemStageVO itemStageVO,Long brokerageStageId) {
        Item item = itemMapper.selectById(itemStageVO.getItemId());
        if (item.getAmount()!=null){
            NonMarketerMarketerBrokerageCalculator calculator = (NonMarketerMarketerBrokerageCalculator) brokerageCalculateSupport.getCalculator(BrokerageRule.LAND_OTHER_RULE);
            ItemBrokerageGeneral general = this.getOne(new LambdaQueryWrapper<ItemBrokerageGeneral>().eq(ItemBrokerageGeneral::getItemId, itemStageVO.getItemId()));
            if (general == null){
                addMarketerBrokerage(itemStageVO.getItemId());
                addParticipatorBrokerage(itemStageVO,brokerageStageId);
                log.info("非正常情况下，更新项目信息时创建了相关的提成记录");
                return;
            }
            Set<ItemStageVO.ParticipatorWeight> participators = itemStageVO.getParticipators();
            Set<ItemBrokerageDetail> participatorBrokerageDetails = participators.stream().map(x -> {
                BrokerageDetailBO brokerageDetailBO = BrokerageDetailBO.builder().build();
                brokerageDetailBO.setBrokerageRule(BrokerageRule.LAND_OTHER_RULE.getCode());
                brokerageDetailBO.setBrokerageMode(BrokerageMode.PERSONAL.getCode());
                brokerageDetailBO.setAdvanceAmount(BigDecimal.ZERO);
                brokerageDetailBO.setPredictAmount(BigDecimal.ZERO);
                brokerageDetailBO.setActualAmount(BigDecimal.ZERO);
                brokerageDetailBO.setAmount(item.getAmount());
                brokerageDetailBO.setUserId(x.getUserId());
                brokerageDetailBO.setGeneralId(general.getId());
                brokerageDetailBO.setBrokerageStageId(brokerageStageId);
                brokerageDetailBO.setStageWeight(itemStageVO.getWeight());
                brokerageDetailBO.setParticipantWeight(x.getWeight());
                ItemBrokerageDetail itemBrokerageDetail = calculator.predictAmount(brokerageDetailBO);
                return itemBrokerageDetail;
            }).collect(Collectors.toSet());
            BigDecimal participatorPredictTotalAmount = participatorBrokerageDetails.stream().map(ItemBrokerageDetail::getPredictAmount).reduce(BigDecimal.ZERO, BigDecimal::add);

            //创建参与人的项目提成详情
            iItemBrokerageDetailService.saveBatch(participatorBrokerageDetails);
            //更新项目提成总表信息
            general.setParticipantPredictBrokerageAmount(participatorPredictTotalAmount);
            general.setParticipantAdvanceStatus(Boolean.TRUE);
            general.setBrokeragePredictAmount(general.getMarketerBrokerageAmount().add(participatorPredictTotalAmount));
            general.setParticipantActualBrokerageAmount(BigDecimal.ZERO);
            general.setStatus(BrokerageState.PREDICTING.getCode());

            this.updateById(general);

        }else {
            log.info("未设置合同金额，创建参与人提成记录终止。itemId[{}]",item.getId());
        }


    }

    @Override
    public void editParticipatorBrokerage(ItemStageVO itemStageVO) {
        iItemBrokerageDetailService.remove(new LambdaQueryWrapper<ItemBrokerageDetail>().eq(ItemBrokerageDetail::getBrokerageStageId,itemStageVO.getId()));
        addParticipatorBrokerage(itemStageVO,itemStageVO.getId());
    }

    @Transactional
    @Override
    public void removeParticipatorBrokerage(Long itemStageId) {
        List<ItemBrokerageDetail> list = iItemBrokerageDetailService.list(new LambdaQueryWrapper<ItemBrokerageDetail>()
                .eq(ItemBrokerageDetail::getBrokerageStageId, itemStageId));
        if (CollectionUtils.isEmpty(list)){
            return ;
        }
        Long generalId = list.get(0).getGeneralId();
        if (generalId == null){
            return ;
        }
        ItemBrokerageGeneral general = this.getById(generalId);
        BigDecimal participatorPredictTotalAmount = list.stream().map(ItemBrokerageDetail::getPredictAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
        general.setParticipantPredictBrokerageAmount(general.getParticipantPredictBrokerageAmount().subtract(participatorPredictTotalAmount));
        general.setBrokeragePredictAmount(general.getBrokeragePredictAmount().subtract(participatorPredictTotalAmount));
       this.updateById(general);
        iItemBrokerageDetailService.remove(new LambdaQueryWrapper<ItemBrokerageDetail>().eq(ItemBrokerageDetail::getBrokerageStageId,itemStageId));
    }

    /**
     * 获取客户经理年回款目标
     * @param userId
     * @return
     */
    private UserTarget getUserTarget(Long userId) {
        UserTarget userTarget = userTargetService.getOne(new LambdaQueryWrapper<UserTarget>().eq(UserTarget::getUserId, userId));
        if (userTarget==null || userTarget.getPaymentTarget()==null){
            userTarget = new UserTarget();
            userTarget.setPaymentTarget(0d);
            log.info("客户经理id[{}]未设置年回款目标。", userId);
        }
        return userTarget;
    }
}
